@@ -1,9 +1,11 @@
 use super::range_wrapper::RangeStartWrapper;
+use crate::range_wrapper::RangeEndWrapper;
 use crate::std_ext::*;
 use alloc::collections::BTreeMap;
+use core::cmp::Ordering;
 use core::fmt::{self, Debug};
 use core::iter::FromIterator;
-use core::ops::Range;
+use core::ops::{Bound, Range};
 use core::prelude::v1::*;
 
 #[cfg(feature = "serde1")]
@@ -26,56 +28,66 @@ pub struct RangeMap<K, V> {
     pub(crate) btm: BTreeMap<RangeStartWrapper<K>, V>,
 }
 
-impl<K, V> Default for RangeMap<K, V>
-where
-    K: Ord + Clone,
-    V: Eq + Clone,
-{
+impl<K, V> Default for RangeMap<K, V> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<K, V> RangeMap<K, V>
+impl<K, V> PartialEq for RangeMap<K, V>
 where
-    K: Ord + Clone,
-    V: Eq + Clone,
+    K: PartialEq,
+    V: PartialEq,
 {
+    fn eq(&self, other: &RangeMap<K, V>) -> bool {
+        self.expanded_iter().eq(other.expanded_iter())
+    }
+}
+
+impl<K, V> PartialOrd for RangeMap<K, V>
+where
+    K: PartialOrd,
+    V: PartialOrd,
+{
+    #[inline]
+    fn partial_cmp(&self, other: &RangeMap<K, V>) -> Option<Ordering> {
+        self.expanded_iter().partial_cmp(other.expanded_iter())
+    }
+}
+
+impl<K, V> Ord for RangeMap<K, V>
+where
+    K: Ord,
+    V: Ord,
+{
+    #[inline]
+    fn cmp(&self, other: &RangeMap<K, V>) -> Ordering {
+        self.expanded_iter().cmp(other.expanded_iter())
+    }
+}
+
+impl<K, V> Eq for RangeMap<K, V>
+where
+    K: Eq,
+    V: Eq,
+{
+}
+
+impl<K, V> RangeMap<K, V> {
     /// Makes a new empty `RangeMap`.
-    pub fn new() -> Self {
+    #[cfg(feature = "const_fn")]
+    pub const fn new() -> Self {
         RangeMap {
             btm: BTreeMap::new(),
         }
     }
 
-    /// Returns a reference to the value corresponding to the given key,
-    /// if the key is covered by any range in the map.
-    pub fn get(&self, key: &K) -> Option<&V> {
-        self.get_key_value(key).map(|(_range, value)| value)
-    }
-
-    /// Returns the range-value pair (as a pair of references) corresponding
-    /// to the given key, if the key is covered by any range in the map.
-    pub fn get_key_value(&self, key: &K) -> Option<(&Range<K>, &V)> {
-        use core::ops::Bound;
-
-        // The only stored range that could contain the given key is the
-        // last stored range whose start is less than or equal to this key.
-        let key_as_start = RangeStartWrapper::new(key.clone()..key.clone());
-        self.btm
-            .range((Bound::Unbounded, Bound::Included(key_as_start)))
-            .next_back()
-            .filter(|(range_start_wrapper, _value)| {
-                // Does the only candidate range contain
-                // the requested key?
-                range_start_wrapper.range.contains(key)
-            })
-            .map(|(range_start_wrapper, value)| (&range_start_wrapper.range, value))
-    }
-
-    /// Returns `true` if any range in the map covers the specified key.
-    pub fn contains_key(&self, key: &K) -> bool {
-        self.get(key).is_some()
+    /// Makes a new empty `RangeMap`.
+    #[cfg(not(feature = "const_fn"))]
+    pub fn new() -> Self {
+        RangeMap {
+            btm: BTreeMap::new(),
+        }
     }
 
     /// Gets an iterator over all pairs of key range and value,
@@ -88,6 +100,111 @@ where
         }
     }
 
+    /// Clears the map, removing all elements.
+    pub fn clear(&mut self) {
+        self.btm.clear();
+    }
+
+    /// Returns the number of elements in the map.
+    pub fn len(&self) -> usize {
+        self.btm.len()
+    }
+
+    /// Returns true if the map contains no elements.
+    pub fn is_empty(&self) -> bool {
+        self.btm.is_empty()
+    }
+
+    /// Returns an iterator that includes both ends of the key range.
+    ///
+    /// Mainly used for comparisons.
+    fn expanded_iter(&self) -> impl Iterator<Item = (&K, &K, &V)> {
+        self.btm.iter().map(|(k, v)| (&k.start, &k.end, v))
+    }
+}
+
+impl<K, V> RangeMap<K, V>
+where
+    K: Ord + Clone,
+{
+    /// Returns a reference to the value corresponding to the given key,
+    /// if the key is covered by any range in the map.
+    pub fn get(&self, key: &K) -> Option<&V> {
+        self.get_key_value(key).map(|(_range, value)| value)
+    }
+
+    /// Returns the range-value pair (as a pair of references) corresponding
+    /// to the given key, if the key is covered by any range in the map.
+    pub fn get_key_value(&self, key: &K) -> Option<(&Range<K>, &V)> {
+        // The only stored range that could contain the given key is the
+        // last stored range whose start is less than or equal to this key.
+        let key_as_start = RangeStartWrapper::new(key.clone()..key.clone());
+        self.btm
+            .range((Bound::Unbounded, Bound::Included(key_as_start)))
+            .next_back()
+            .filter(|(start_wrapper, _value)| {
+                // Does the only candidate range contain
+                // the requested key?
+                start_wrapper.end_wrapper.range.contains(key)
+            })
+            .map(|(start_wrapper, value)| (&start_wrapper.end_wrapper.range, value))
+    }
+
+    /// Returns `true` if any range in the map covers the specified key.
+    pub fn contains_key(&self, key: &K) -> bool {
+        self.get(key).is_some()
+    }
+
+    /// Gets an iterator over all the maximally-sized ranges
+    /// contained in `outer_range` that are not covered by
+    /// any range stored in the map.
+    ///
+    /// If the start and end of the outer range are the same
+    /// and it does not overlap any stored range, then a single
+    /// empty gap will be returned.
+    ///
+    /// The iterator element type is `Range<K>`.
+    pub fn gaps<'a>(&'a self, outer_range: &'a Range<K>) -> Gaps<'a, K, V> {
+        Gaps {
+            outer_range,
+            keys: self.btm.keys(),
+            // We'll start the candidate range at the start of the outer range
+            // without checking what's there. Each time we yield an item,
+            // we'll skip any ranges we find before the next gap.
+            candidate_start: &outer_range.start,
+        }
+    }
+
+    /// Gets an iterator over all the stored ranges that are
+    /// either partially or completely overlapped by the given range.
+    pub fn overlapping<'a>(&'a self, range: &'a Range<K>) -> Overlapping<K, V> {
+        // Find the first matching stored range by its _end_,
+        // using sneaky layering and `Borrow` implementation. (See `range_wrappers` module.)
+        let start_sliver = RangeEndWrapper::new(range.start.clone()..range.start.clone());
+        let btm_range_iter = self
+            .btm
+            .range::<RangeEndWrapper<K>, (Bound<&RangeEndWrapper<K>>, Bound<_>)>((
+                Bound::Excluded(&start_sliver),
+                Bound::Unbounded,
+            ));
+        Overlapping {
+            query_range: range,
+            btm_range_iter,
+        }
+    }
+
+    /// Returns `true` if any range in the map completely or partially
+    /// overlaps the given range.
+    pub fn overlaps(&self, range: &Range<K>) -> bool {
+        self.overlapping(range).next().is_some()
+    }
+}
+
+impl<K, V> RangeMap<K, V>
+where
+    K: Ord + Clone,
+    V: Eq + Clone,
+{
     /// Insert a pair of key range and value into the map.
     ///
     /// If the inserted range partially or completely overlaps any
@@ -102,8 +219,6 @@ where
     ///
     /// Panics if range `start >= end`.
     pub fn insert(&mut self, range: Range<K>, value: V) {
-        use core::ops::Bound;
-
         // We don't want to have to make empty ranges make sense;
         // they don't represent anything meaningful in this structure.
         assert!(range.start < range.end);
@@ -111,7 +226,7 @@ where
         // Wrap up the given range so that we can "borrow"
         // it as a wrapper reference to either its start or end.
         // See `range_wrapper.rs` for explanation of these hacks.
-        let mut new_range_start_wrapper: RangeStartWrapper<K> = RangeStartWrapper::new(range);
+        let mut new_start_wrapper: RangeStartWrapper<K> = RangeStartWrapper::new(range);
         let new_value = value;
 
         // Is there a stored range either overlapping the start of
@@ -122,29 +237,32 @@ where
         // or the one before that if both of the above cases exist.
         let mut candidates = self
             .btm
-            .range((Bound::Unbounded, Bound::Included(&new_range_start_wrapper)))
+            .range::<RangeStartWrapper<K>, (Bound<&RangeStartWrapper<K>>, Bound<&RangeStartWrapper<K>>)>((
+                Bound::Unbounded,
+                Bound::Included(&new_start_wrapper),
+            ))
             .rev()
             .take(2)
-            .filter(|(stored_range_start_wrapper, _stored_value)| {
+            .filter(|(stored_start_wrapper, _stored_value)| {
                 // Does the candidate range either overlap
                 // or immediately precede the range to insert?
                 // (Remember that it might actually cover the _whole_
                 // range to insert and then some.)
-                stored_range_start_wrapper
+                stored_start_wrapper
+                    .end_wrapper
                     .range
-                    .touches(&new_range_start_wrapper.range)
+                    .touches(&new_start_wrapper.end_wrapper.range)
             });
         if let Some(mut candidate) = candidates.next() {
             // Or the one before it if both cases described above exist.
             if let Some(another_candidate) = candidates.next() {
                 candidate = another_candidate;
             }
-            let (stored_range_start_wrapper, stored_value) =
-                (candidate.0.clone(), candidate.1.clone());
+            let (stored_start_wrapper, stored_value) = (candidate.0.clone(), candidate.1.clone());
             self.adjust_touching_ranges_for_insert(
-                stored_range_start_wrapper,
+                stored_start_wrapper,
                 stored_value,
-                &mut new_range_start_wrapper.range,
+                &mut new_start_wrapper.end_wrapper.range,
                 &new_value,
             );
         }
@@ -162,12 +280,13 @@ where
         // REVISIT: Possible micro-optimisation: `impl Borrow<T> for RangeStartWrapper<T>`
         // and use that to search here, to avoid constructing another `RangeStartWrapper`.
         let new_range_end_as_start = RangeStartWrapper::new(
-            new_range_start_wrapper.range.end.clone()..new_range_start_wrapper.range.end.clone(),
+            new_start_wrapper.end_wrapper.range.end.clone()
+                ..new_start_wrapper.end_wrapper.range.end.clone(),
         );
-        while let Some((stored_range_start_wrapper, stored_value)) = self
+        while let Some((stored_start_wrapper, stored_value)) = self
             .btm
-            .range((
-                Bound::Included(&new_range_start_wrapper),
+            .range::<RangeStartWrapper<K>, (Bound<&RangeStartWrapper<K>>, Bound<&RangeStartWrapper<K>>)>((
+                Bound::Included(&new_start_wrapper),
                 Bound::Included(&new_range_end_as_start),
             ))
             .next()
@@ -176,7 +295,8 @@ where
             // and the stored range starts at the end of the range to insert,
             // then we don't want to keep looping forever trying to find more!
             #[allow(clippy::suspicious_operation_groupings)]
-            if stored_range_start_wrapper.range.start == new_range_start_wrapper.range.end
+            if stored_start_wrapper.end_wrapper.range.start
+                == new_start_wrapper.end_wrapper.range.end
                 && *stored_value != new_value
             {
                 // We're beyond the last stored range that could be relevant.
@@ -187,19 +307,19 @@ where
                 break;
             }
 
-            let stored_range_start_wrapper = stored_range_start_wrapper.clone();
+            let stored_start_wrapper = stored_start_wrapper.clone();
             let stored_value = stored_value.clone();
 
             self.adjust_touching_ranges_for_insert(
-                stored_range_start_wrapper,
+                stored_start_wrapper,
                 stored_value,
-                &mut new_range_start_wrapper.range,
+                &mut new_start_wrapper.end_wrapper.range,
                 &new_value,
             );
         }
 
         // Insert the (possibly expanded) new range, and we're done!
-        self.btm.insert(new_range_start_wrapper, new_value);
+        self.btm.insert(new_start_wrapper, new_value);
     }
 
     /// Removes a range from the map, if all or any of it was present.
@@ -213,35 +333,36 @@ where
     ///
     /// Panics if range `start >= end`.
     pub fn remove(&mut self, range: Range<K>) {
-        use core::ops::Bound;
-
         // We don't want to have to make empty ranges make sense;
         // they don't represent anything meaningful in this structure.
         assert!(range.start < range.end);
 
-        let range_start_wrapper: RangeStartWrapper<K> = RangeStartWrapper::new(range);
-        let range = &range_start_wrapper.range;
+        let start_wrapper: RangeStartWrapper<K> = RangeStartWrapper::new(range);
+        let range = &start_wrapper.end_wrapper.range;
 
         // Is there a stored range overlapping the start of
         // the range to insert?
         //
         // If there is any such stored range, it will be the last
         // whose start is less than or equal to the start of the range to insert.
-        if let Some((stored_range_start_wrapper, stored_value)) = self
+        if let Some((stored_start_wrapper, stored_value)) = self
             .btm
-            .range((Bound::Unbounded, Bound::Included(&range_start_wrapper)))
+            .range::<RangeStartWrapper<K>, (Bound<&RangeStartWrapper<K>>, Bound<&RangeStartWrapper<K>>)>((Bound::Unbounded, Bound::Included(&start_wrapper)))
             .next_back()
-            .filter(|(stored_range_start_wrapper, _stored_value)| {
+            .filter(|(stored_start_wrapper, _stored_value)| {
                 // Does the only candidate range overlap
                 // the range to insert?
-                stored_range_start_wrapper.range.overlaps(range)
+                stored_start_wrapper
+                    .end_wrapper
+                    .range
+                    .overlaps(range)
             })
-            .map(|(stored_range_start_wrapper, stored_value)| {
-                (stored_range_start_wrapper.clone(), stored_value.clone())
+            .map(|(stored_start_wrapper, stored_value)| {
+                (stored_start_wrapper.clone(), stored_value.clone())
             })
         {
             self.adjust_overlapping_ranges_for_remove(
-                stored_range_start_wrapper,
+                stored_start_wrapper,
                 stored_value,
                 range,
             );
@@ -256,19 +377,19 @@ where
         // REVISIT: Possible micro-optimisation: `impl Borrow<T> for RangeStartWrapper<T>`
         // and use that to search here, to avoid constructing another `RangeStartWrapper`.
         let new_range_end_as_start = RangeStartWrapper::new(range.end.clone()..range.end.clone());
-        while let Some((stored_range_start_wrapper, stored_value)) = self
+        while let Some((stored_start_wrapper, stored_value)) = self
             .btm
-            .range((
-                Bound::Excluded(&range_start_wrapper),
+            .range::<RangeStartWrapper<K>, (Bound<&RangeStartWrapper<K>>, Bound<&RangeStartWrapper<K>>)>((
+                Bound::Excluded(&start_wrapper),
                 Bound::Excluded(&new_range_end_as_start),
             ))
             .next()
-            .map(|(stored_range_start_wrapper, stored_value)| {
-                (stored_range_start_wrapper.clone(), stored_value.clone())
+            .map(|(stored_start_wrapper, stored_value)| {
+                (stored_start_wrapper.clone(), stored_value.clone())
             })
         {
             self.adjust_overlapping_ranges_for_remove(
-                stored_range_start_wrapper,
+                stored_start_wrapper,
                 stored_value,
                 range,
             );
@@ -277,7 +398,7 @@ where
 
     fn adjust_touching_ranges_for_insert(
         &mut self,
-        stored_range_start_wrapper: RangeStartWrapper<K>,
+        stored_start_wrapper: RangeStartWrapper<K>,
         stored_value: V,
         new_range: &mut Range<K>,
         new_value: &V,
@@ -291,31 +412,30 @@ where
             // This means that no matter how big or where the stored range is,
             // we will expand the new range's bounds to subsume it,
             // and then delete the stored range.
-            new_range.start =
-                min(&new_range.start, &stored_range_start_wrapper.range.start).clone();
-            new_range.end = max(&new_range.end, &stored_range_start_wrapper.range.end).clone();
-            self.btm.remove(&stored_range_start_wrapper);
+            new_range.start = min(&new_range.start, &stored_start_wrapper.start).clone();
+            new_range.end = max(&new_range.end, &stored_start_wrapper.end).clone();
+            self.btm.remove(&stored_start_wrapper);
         } else {
             // The ranges have different values.
-            if new_range.overlaps(&stored_range_start_wrapper.range) {
+            if new_range.overlaps(&stored_start_wrapper.range) {
                 // The ranges overlap. This is a little bit more complicated.
                 // Delete the stored range, and then add back between
                 // 0 and 2 subranges at the ends of the range to insert.
-                self.btm.remove(&stored_range_start_wrapper);
-                if stored_range_start_wrapper.range.start < new_range.start {
+                self.btm.remove(&stored_start_wrapper);
+                if stored_start_wrapper.start < new_range.start {
                     // Insert the piece left of the range to insert.
                     self.btm.insert(
                         RangeStartWrapper::new(
-                            stored_range_start_wrapper.range.start..new_range.start.clone(),
+                            stored_start_wrapper.end_wrapper.range.start..new_range.start.clone(),
                         ),
                         stored_value.clone(),
                     );
                 }
-                if stored_range_start_wrapper.range.end > new_range.end {
+                if stored_start_wrapper.end_wrapper.range.end > new_range.end {
                     // Insert the piece right of the range to insert.
                     self.btm.insert(
                         RangeStartWrapper::new(
-                            new_range.end.clone()..stored_range_start_wrapper.range.end,
+                            new_range.end.clone()..stored_start_wrapper.end_wrapper.range.end,
                         ),
                         stored_value,
                     );
@@ -329,50 +449,29 @@ where
 
     fn adjust_overlapping_ranges_for_remove(
         &mut self,
-        stored_range_start_wrapper: RangeStartWrapper<K>,
+        stored: RangeStartWrapper<K>,
         stored_value: V,
         range_to_remove: &Range<K>,
     ) {
         // Delete the stored range, and then add back between
         // 0 and 2 subranges at the ends of the range to insert.
-        self.btm.remove(&stored_range_start_wrapper);
-        let stored_range = stored_range_start_wrapper.range;
+        self.btm.remove(&stored);
+        let stored_range = stored.end_wrapper;
         if stored_range.start < range_to_remove.start {
             // Insert the piece left of the range to insert.
             self.btm.insert(
-                RangeStartWrapper::new(stored_range.start..range_to_remove.start.clone()),
+                RangeStartWrapper::new(stored_range.range.start..range_to_remove.start.clone()),
                 stored_value.clone(),
             );
         }
-        if stored_range.end > range_to_remove.end {
+        if stored_range.range.end > range_to_remove.end {
             // Insert the piece right of the range to insert.
             self.btm.insert(
-                RangeStartWrapper::new(range_to_remove.end.clone()..stored_range.end),
+                RangeStartWrapper::new(range_to_remove.end.clone()..stored_range.range.end),
                 stored_value,
             );
         }
     }
-
-    /// Gets an iterator over all the maximally-sized ranges
-    /// contained in `outer_range` that are not covered by
-    /// any range stored in the map.
-    ///
-    /// If the start and end of the outer range are the same
-    /// and it does not overlap any stored range, then a single
-    /// empty gap will be returned.
-    ///
-    /// The iterator element type is `Range<K>`.
-    pub fn gaps<'a>(&'a self, outer_range: &'a Range<K>) -> Gaps<'a, K, V> {
-        let keys = self.btm.keys().peekable();
-        Gaps {
-            outer_range,
-            keys,
-            // We'll start the candidate range at the start of the outer range
-            // without checking what's there. Each time we yield an item,
-            // we'll skip any ranges we find before the next gap.
-            candidate_start: &outer_range.start,
-        }
-    }
 }
 
 /// An iterator over the entries of a `RangeMap`, ordered by key range.
@@ -395,7 +494,9 @@ where
     type Item = (&'a Range<K>, &'a V);
 
     fn next(&mut self) -> Option<(&'a Range<K>, &'a V)> {
-        self.inner.next().map(|(by_start, v)| (&by_start.range, v))
+        self.inner
+            .next()
+            .map(|(by_start, v)| (&by_start.end_wrapper.range, v))
     }
 
     fn size_hint(&self) -> (usize, Option<usize>) {
@@ -428,7 +529,9 @@ impl<K, V> IntoIterator for RangeMap<K, V> {
 impl<K, V> Iterator for IntoIter<K, V> {
     type Item = (Range<K>, V);
     fn next(&mut self) -> Option<(Range<K>, V)> {
-        self.inner.next().map(|(by_start, v)| (by_start.range, v))
+        self.inner
+            .next()
+            .map(|(by_start, v)| (by_start.end_wrapper.range, v))
     }
     fn size_hint(&self) -> (usize, Option<usize>) {
         self.inner.size_hint()
@@ -438,11 +541,7 @@ impl<K, V> Iterator for IntoIter<K, V> {
 // We can't just derive this automatically, because that would
 // expose irrelevant (and private) implementation details.
 // Instead implement it in the same way that the underlying BTreeMap does.
-impl<K: Debug, V: Debug> Debug for RangeMap<K, V>
-where
-    K: Ord + Clone,
-    V: Eq + Clone,
-{
+impl<K: Debug, V: Debug> Debug for RangeMap<K, V> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_map().entries(self.iter()).finish()
     }
@@ -475,8 +574,8 @@ where
 #[cfg(feature = "serde1")]
 impl<K, V> Serialize for RangeMap<K, V>
 where
-    K: Ord + Clone + Serialize,
-    V: Eq + Clone + Serialize,
+    K: Serialize,
+    V: Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -553,7 +652,7 @@ where
 /// [`gaps`]: RangeMap::gaps
 pub struct Gaps<'a, K, V> {
     outer_range: &'a Range<K>,
-    keys: core::iter::Peekable<alloc::collections::btree_map::Keys<'a, RangeStartWrapper<K>, V>>,
+    keys: alloc::collections::btree_map::Keys<'a, RangeStartWrapper<K>, V>,
     candidate_start: &'a K,
 }
 
@@ -567,36 +666,24 @@ where
     type Item = Range<K>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        // Skip past any ranges to find the first value that might
-        // be the start of a gap.
-        while let Some(item) = self.keys.peek() {
-            if item.range.start <= *self.candidate_start {
-                // Next potential gap starts after this range, and at least as
-                // late as the start of the outer range.
-                self.candidate_start = &item.range.end;
-                // Adjust the start to at least the start of the outer range.
-                if *self.candidate_start < self.outer_range.start {
-                    self.candidate_start = &self.outer_range.start;
-                }
-                let _ = self.keys.next();
-            } else {
-                // There's a gap before this item!
-                let gap = if item.range.start <= self.outer_range.end {
-                    // It starts within the outer range; return the gap directly.
-                    self.candidate_start.clone()..item.range.start.clone()
-                } else {
-                    // Next item starts beyond the end of the outer range;
-                    // end our gap at the end of the outer range.
-                    self.candidate_start.clone()..self.outer_range.end.clone()
-                };
-                // Next potential gap starts after this range.
-                self.candidate_start = &item.range.end;
+        for item in &mut self.keys {
+            let range = &item.range;
+            if range.end <= *self.candidate_start {
+                // We're already completely past it; ignore it.
+            } else if range.start <= *self.candidate_start {
+                // We're inside it; move past it.
+                self.candidate_start = &range.end;
+            } else if range.start < self.outer_range.end {
+                // It starts before the end of the outer range,
+                // so move past it and then yield a gap.
+                let gap = self.candidate_start.clone()..range.start.clone();
+                self.candidate_start = &range.end;
                 return Some(gap);
             }
         }
 
-        // If we got this far, the only other gap that might be
-        // is at the end of the outer range.
+        // Now that we've run out of items, the only other possible
+        // gap is at the end of the outer range.
         if *self.candidate_start < self.outer_range.end {
             // There's a gap at the end!
             let gap = self.candidate_start.clone()..self.outer_range.end.clone();
@@ -610,6 +697,46 @@ where
     }
 }
 
+/// An iterator over all stored ranges partially or completely
+/// overlapped by a given range.
+///
+/// The iterator element type is `(&'a Range<K>, &'a V)`.
+///
+/// This `struct` is created by the [`overlapping`] method on [`RangeMap`]. See its
+/// documentation for more.
+///
+/// [`overlapping`]: RangeMap::overlapping
+pub struct Overlapping<'a, K, V> {
+    query_range: &'a Range<K>,
+    btm_range_iter: alloc::collections::btree_map::Range<'a, RangeStartWrapper<K>, V>,
+}
+
+// `Overlapping` is always fused. (See definition of `next` below.)
+impl<'a, K, V> core::iter::FusedIterator for Overlapping<'a, K, V> where K: Ord {}
+
+impl<'a, K, V> Iterator for Overlapping<'a, K, V>
+where
+    K: Ord,
+{
+    type Item = (&'a Range<K>, &'a V);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if let Some((k, v)) = self.btm_range_iter.next() {
+            if k.start < self.query_range.end {
+                Some((&k.range, v))
+            } else {
+                // The rest of the items in the underlying iterator
+                // are past the query range. We can keep taking items
+                // from that iterator and this will remain true,
+                // so this is enough to make the iterator fused.
+                None
+            }
+        } else {
+            None
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1131,10 +1258,9 @@ mod tests {
         // ◌ ◌ ◌ ◌ ◆ ◌ ◌ ◌ ◌ ◌
         let outer_range = 4..4;
         let mut gaps = range_map.gaps(&outer_range);
-        // Should yield a gap covering the zero-width outer range.
-        assert_eq!(gaps.next(), Some(4..4));
-        // Gaps iterator should be fused.
+        // Should not yield any gaps, because a zero-width outer range covers no values.
         assert_eq!(gaps.next(), None);
+        // Gaps iterator should be fused.
         assert_eq!(gaps.next(), None);
     }
 
@@ -1218,6 +1344,99 @@ mod tests {
         assert_eq!(gaps.next(), None);
     }
 
+    // This test fails in v1.0.2
+    #[test]
+    fn outer_range_lies_within_first_of_two_stored_ranges() {
+        let mut range_map: RangeMap<u64, ()> = RangeMap::new();
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◆----------◇ ◌ ◌ ◌ ◌
+        range_map.insert(0..5, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌◌ ◌ ◆---◇ ◌
+        range_map.insert(6..8, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆--◇ ◌  ◌  ◌  ◌  ◌
+        let outer_range: Range<u64> = 1..3;
+        let mut gaps = range_map.gaps(&outer_range);
+        assert_eq!(gaps.next(), None);
+    }
+
+    // Overlapping tests
+
+    #[test]
+    fn overlapping_with_empty_map() {
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        let range_map: RangeMap<u32, ()> = RangeMap::new();
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆-------------◇ ◌
+        let query_range = 1..8;
+        let mut overlapping = range_map.overlapping(&query_range);
+        // Should not yield any items.
+        assert_eq!(overlapping.next(), None);
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+    }
+
+    #[test]
+    fn overlapping_partial_edges_complete_middle() {
+        let mut range_map: RangeMap<u32, ()> = RangeMap::new();
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ●---◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(0..2, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ●-◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(3..4, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ●---◌ ◌ ◌
+        range_map.insert(5..7, ());
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆---------◇ ◌ ◌ ◌
+        let query_range = 1..6;
+
+        let mut overlapping = range_map.overlapping(&query_range);
+
+        // Should yield partially overlapped range at start.
+        assert_eq!(overlapping.next(), Some((&(0..2), &())));
+        // Should yield completely overlapped range in middle.
+        assert_eq!(overlapping.next(), Some((&(3..4), &())));
+        // Should yield partially overlapped range at end.
+        assert_eq!(overlapping.next(), Some((&(5..7), &())));
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+        assert_eq!(overlapping.next(), None);
+    }
+
+    #[test]
+    fn overlapping_non_overlapping_edges_complete_middle() {
+        let mut range_map: RangeMap<u32, ()> = RangeMap::new();
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ●---◌ ◌ ◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(0..2, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ●-◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(3..4, ());
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ●---◌ ◌ ◌
+        range_map.insert(5..7, ());
+
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◆-----◇ ◌ ◌ ◌ ◌
+        let query_range = 2..5;
+
+        let mut overlapping = range_map.overlapping(&query_range);
+
+        // Should only yield the completely overlapped range in middle.
+        // (Not the ranges that are touched by not covered to either side.)
+        assert_eq!(overlapping.next(), Some((&(3..4), &())));
+        // Gaps iterator should be fused.
+        assert_eq!(overlapping.next(), None);
+        assert_eq!(overlapping.next(), None);
+    }
+
     ///
     /// impl Debug
     ///
@@ -1258,6 +1477,55 @@ mod tests {
         assert_eq!(cloned, consumed);
     }
 
+    // Equality
+    #[test]
+    fn eq() {
+        let mut a: RangeMap<u32, bool> = RangeMap::new();
+        a.insert(1..3, false);
+
+        let mut b: RangeMap<u32, bool> = RangeMap::new();
+        b.insert(1..4, false);
+
+        let mut c: RangeMap<u32, bool> = RangeMap::new();
+        c.insert(1..3, false);
+
+        assert_ne!(a, b);
+        assert_ne!(b, a);
+
+        assert_eq!(a, c);
+        assert_eq!(c, a);
+        assert_eq!(a, a);
+    }
+
+    // Ord
+    #[test]
+    fn partial_ord() {
+        let mut a: RangeMap<u32, bool> = RangeMap::new();
+        a.insert(1..3, false);
+
+        let mut b: RangeMap<u32, bool> = RangeMap::new();
+        b.insert(1..4, false);
+
+        assert_eq!(a.partial_cmp(&a), Some(Ordering::Equal));
+
+        assert_eq!(a.partial_cmp(&b), Some(Ordering::Less));
+        assert_eq!(b.partial_cmp(&a), Some(Ordering::Greater));
+    }
+
+    #[test]
+    fn ord() {
+        let mut a: RangeMap<u32, bool> = RangeMap::new();
+        a.insert(1..3, false);
+
+        let mut b: RangeMap<u32, bool> = RangeMap::new();
+        b.insert(1..4, false);
+
+        assert_eq!(a.cmp(&a), Ordering::Equal);
+
+        assert_eq!(a.cmp(&b), Ordering::Less);
+        assert_eq!(b.cmp(&a), Ordering::Greater);
+    }
+
     // impl Serialize
 
     #[cfg(feature = "serde1")]
@@ -1285,4 +1553,9 @@ mod tests {
         let reserialized = serde_json::to_string(&range_map).expect("Failed to re-serialize");
         assert_eq!(reserialized, input);
     }
+
+    // const fn
+
+    #[cfg(feature = "const_fn")]
+    const _MAP: RangeMap<u32, bool> = RangeMap::new();
 }
